"From: https://github.com/bazelbuild/rules_nodejs/blob/master/examples/jest/jest.bzl"

load("@npm//:jest/package_json.bzl", "bin")

def jest_test(name, srcs, deps = [], size = "medium", jest_config = "//:jest.config", snapshots = [], flaky = False, additional_args = [], tags = [], **kwargs):
    """A macro around the autogenerated jest_test rule.

    Args:
        name: target name
        srcs: list of tests, srcs & snapshot files
        deps: npm deps
        size: test size
        snapshots: snapshot files
        jest_config: jest.config.js file, default to the root one
        flaky: Whether this test is flaky
        additional_args: Any addl args
        tags: tags
        **kwargs: the rest
    """
    args = [
        "--no-cache",
        "--no-watchman",
        "--ci",
        "--runInBand",
        "--colors",
    ] + additional_args
    args.extend(["--config", "$(rootpath %s)" % jest_config])
    for src in srcs:
        args.extend(["--runTestsByPath", "$(rootpath %s)" % src])

    data = [jest_config] + srcs + snapshots + deps + [
        "//:node_modules/@jest/transform",
        "//:node_modules/ts-jest",
        "//:node_modules/@types/jest",
        "//:node_modules/tslib",
        "//:tsconfig",
        "//tools:jest-reporter",
    ]

    bin.jest_test(
        name = name,
        data = data,
        args = args,
        tags = [
            # Need to set the pwd to avoid jest needing a runfiles helper
            # Windows users with permissions can use --enable_runfiles
            # to make this test work
            "no-bazelci-windows",
            # TODO: why does this fail almost all the time, but pass on local Mac?
            "no-bazelci-mac",
        ] + tags,
        size = size,
        flaky = flaky,
        **kwargs
    )

    # This target is used to update the snapshot
    # e.g bazel run //packages/react-intl:unit.update
    bin.jest_binary(
        name = "%s.update" % name,
        data = data,
        args = args + ["-u"],
        chdir = "$$BUILD_WORKSPACE_DIRECTORY",
        tags = [
            # Need to set the pwd to avoid jest needing a runfiles helper
            # Windows users with permissions can use --enable_runfiles
            # to make this test work
            "no-bazelci-windows",
            # TODO: why does this fail almost all the time, but pass on local Mac?
            "no-bazelci-mac",
        ] + tags,
        **kwargs
    )
